@@ -39,6 +39,7 @@ struct _EShortcutModelItem
 {
 	gchar *name;
 	gchar *url;
+	GdkPixbuf *image;
 };
 
 
@@ -70,7 +71,8 @@ static void e_shortcut_model_real_add_item	(EShortcutModel	*shortcut_model,
 						 gint		 group_num,
 						 gint		 position,
 						 const gchar	*item_url,
-						 const gchar	*item_name);
+						 const gchar	*item_name,
+						 GdkPixbuf      *item_image);
 static void e_shortcut_model_real_remove_item	(EShortcutModel	 *shortcut_model,
 						 gint		  group_num,
 						 gint		  item_num);
@@ -79,7 +81,8 @@ static void e_shortcut_model_real_update_item   (EShortcutModel *shortcut_model,
 						 gint            group_num,
 						 gint            item_num,
 						 const gchar    *item_url,
-						 const gchar    *item_name);
+						 const gchar    *item_name,
+						 GdkPixbuf      *item_image);
 static void e_shortcut_model_marshal1		(GtkObject	*object,
 						 GtkSignalFunc	 func,
 						 gpointer	 func_data,
@@ -98,7 +101,8 @@ static void e_shortcut_model_real_get_item_info	(EShortcutModel *shortcut_model,
 						 gint		 group_num,
 						 gint		 item_num,
 						 gchar	       **item_url,
-						 gchar	       **item_name);
+						 gchar	       **item_name,
+						 GdkPixbuf     **item_image);
 
 enum
 {
@@ -157,9 +161,9 @@ e_shortcut_model_class_init	(EShortcutModelClass	*class)
 				GTK_SIGNAL_OFFSET (EShortcutModelClass,
 						   item_added),
 				e_shortcut_model_marshal2,
-				GTK_TYPE_NONE, 4,
+				GTK_TYPE_NONE, 5,
 				GTK_TYPE_INT, GTK_TYPE_INT,
-				GTK_TYPE_STRING, GTK_TYPE_STRING);
+				GTK_TYPE_STRING, GTK_TYPE_STRING, GTK_TYPE_POINTER);
 
 	e_shortcut_model_signals[ITEM_REMOVED] =
 		gtk_signal_new ("item_removed",
@@ -178,9 +182,9 @@ e_shortcut_model_class_init	(EShortcutModelClass	*class)
 				GTK_SIGNAL_OFFSET (EShortcutModelClass,
 						   item_updated),
 				e_shortcut_model_marshal2,
-				GTK_TYPE_NONE, 4,
+				GTK_TYPE_NONE, 5,
 				GTK_TYPE_INT, GTK_TYPE_INT,
-				GTK_TYPE_STRING, GTK_TYPE_STRING);
+				GTK_TYPE_STRING, GTK_TYPE_STRING, GTK_TYPE_POINTER);
 
 	gtk_object_class_add_signals (object_class, e_shortcut_model_signals,
 				      LAST_SIGNAL);
@@ -335,7 +339,8 @@ e_shortcut_model_add_item	(EShortcutModel	*shortcut_model,
 				 gint		 group_num,
 				 gint		 position,
 				 const gchar	*item_url,
-				 const gchar	*item_name)
+				 const gchar	*item_name,
+				 GdkPixbuf      *item_image)
 {
 	EShortcutModelGroup *group;
 
@@ -353,7 +358,7 @@ e_shortcut_model_add_item	(EShortcutModel	*shortcut_model,
 
 	gtk_signal_emit (GTK_OBJECT (shortcut_model),
 			 e_shortcut_model_signals[ITEM_ADDED],
-			 group_num, position, item_url, item_name);
+			 group_num, position, item_url, item_name, item_image);
 
 	return position;
 }
@@ -364,7 +369,8 @@ e_shortcut_model_real_add_item	(EShortcutModel	*shortcut_model,
 				 gint		 group_num,
 				 gint		 item_num,
 				 const gchar	*item_url,
-				 const gchar	*item_name)
+				 const gchar	*item_name,
+				 GdkPixbuf      *item_image)
 {
 	EShortcutModelGroup *group;
 	EShortcutModelItem *item, tmp_item;
@@ -388,6 +394,11 @@ e_shortcut_model_real_add_item	(EShortcutModel	*shortcut_model,
 
 	item->name = g_strdup (item_name);
 	item->url = g_strdup (item_url);
+
+	if (item_image != NULL)
+		item->image = gdk_pixbuf_ref (item_image);
+	else
+		item->image = NULL;
 }
 
 
@@ -434,7 +445,8 @@ e_shortcut_model_real_update_item (EShortcutModel *shortcut_model,
 				   gint            group_num,
 				   gint            item_num,
 				   const gchar    *item_url,
-				   const gchar    *item_name)
+				   const gchar    *item_name,
+				   GdkPixbuf      *item_image)
 {
 	EShortcutModelGroup *group;
 	EShortcutModelItem *item;
@@ -456,16 +468,26 @@ e_shortcut_model_real_update_item (EShortcutModel *shortcut_model,
 
 	g_free (item->name);
 	g_free (item->url);
+
+	if (item->image != NULL)
+		gdk_pixbuf_unref (item->image);
+
 	item->name = g_strdup (item_name);
 	item->url = g_strdup (item_url);
+
+	if (item->image != NULL)
+		item->image = gdk_pixbuf_ref (item_image);
+	else
+		item->image = NULL;
 }
 
 void
 e_shortcut_model_update_item (EShortcutModel *shortcut_model,
-			      gint group_num,
-			      gint item_num,
-			      const gchar *item_url,
-			      const gchar *item_name)
+			      gint            group_num,
+			      gint            item_num,
+			      const gchar    *item_url,
+			      const gchar    *item_name,
+			      GdkPixbuf      *item_image)
 {
 
 	g_return_if_fail (E_IS_SHORTCUT_MODEL (shortcut_model));
@@ -476,7 +498,7 @@ e_shortcut_model_update_item (EShortcutModel *shortcut_model,
 			  
 	gtk_signal_emit (GTK_OBJECT (shortcut_model),
 			 e_shortcut_model_signals[ITEM_UPDATED],
-			 group_num, item_num, item_url, item_name);
+			 group_num, item_num, item_url, item_name, item_image);
 }
 
 static void
@@ -498,11 +520,11 @@ e_shortcut_model_marshal2	(GtkObject	*object,
 				 gpointer	 func_data,
 				 GtkArg		*args)
 {
-	void (*rfunc) (GtkObject*, gint, gint, gpointer, gpointer, gpointer);
-	rfunc = (void (*) (GtkObject*, gint, gint, gpointer, gpointer,
+	void (*rfunc) (GtkObject*, gint, gint, gpointer, gpointer, gpointer, gpointer);
+	rfunc = (void (*) (GtkObject*, gint, gint, gpointer, gpointer, gpointer,
 			   gpointer)) func;
 	(*rfunc) (object, GTK_VALUE_INT (args[0]), GTK_VALUE_INT (args[1]),
-		  GTK_VALUE_STRING (args[2]), GTK_VALUE_STRING (args[3]),
+		  GTK_VALUE_STRING (args[2]), GTK_VALUE_STRING (args[3]), GTK_VALUE_POINTER (args[4]),
 		  func_data);
 }
 
@@ -582,13 +604,14 @@ e_shortcut_model_get_item_info	(EShortcutModel *shortcut_model,
 				 gint		 group_num,
 				 gint		 item_num,
 				 gchar	       **item_url,
-				 gchar	       **item_name)
+				 gchar	       **item_name,
+				 GdkPixbuf     **item_image)
 {
 	g_return_if_fail (E_IS_SHORTCUT_MODEL (shortcut_model));
 
 	ESM_CLASS (shortcut_model)->get_item_info (shortcut_model,
 						   group_num, item_num,
-						   item_url, item_name);
+						   item_url, item_name, item_image);
 }
 
 
@@ -597,7 +620,8 @@ e_shortcut_model_real_get_item_info	(EShortcutModel *shortcut_model,
 					 gint		 group_num,
 					 gint		 item_num,
 					 gchar	       **item_url,
-					 gchar	       **item_name)
+					 gchar	       **item_name,
+					 GdkPixbuf     **item_image)
 {
 	EShortcutModelGroup *group;
 	EShortcutModelItem *item;
@@ -619,5 +643,8 @@ e_shortcut_model_real_get_item_info	(EShortcutModel *shortcut_model,
 
 	if (item_name)
 		*item_name = g_strdup (item->name);
+
+	if (item_image)
+		*item_image = gdk_pixbuf_ref (item->image);
 }
 
